@@ -1,5 +1,6 @@
 from sudachipy import dictionary
 from sudachipy import tokenizer
+from sudachipy.plugin import oov
 from kuro2sudachi.normalizer import SudachiCharNormalizer
 import jaconv
 import fileinput
@@ -28,14 +29,17 @@ parser.add_argument(
     help="rewrite text file path",
 )
 parser.add_argument(
-    "-e", "--rm_already_exist", help="remove words system dict already exist"
+    "--rm_already_exist",
+    action="store_true",
+    help="remove words system dict already exist"
 )
 parser.add_argument("-r", "--sudachi_setting", help="the setting file in JSON format")
 parser.add_argument("-s", "--sudachi_dict_type", help="sudachidict type")
+parser.add_argument("-m", "--merge_dict", help="A dictionary for split registration of words that are not in the system dictionary. Must be specified as a user dictionary in sudachi's configuration file (json).")
 parser.add_argument(
     "--ignore",
     action="store_true",
-    help="ignore invalid format line or unsupported pos error",
+    help="ignore invalid format line / unsupported pos error / oov error in splitted word",
 )
 
 default_setting = {
@@ -68,6 +72,10 @@ class DictFormatError(Error):
     pass
 
 
+class OOVError(Error):
+    pass
+
+
 class Converter:
     def __init__(
         self,
@@ -106,13 +114,20 @@ class Converter:
             raise DictFormatError(f"'{line}' is invalid format")
 
         words = [m.surface() for m in self.tokenizer.tokenize(word, mode)]
+
+        # alrady exists in system dic
         if self.rm and len(words) == 1:
             return ""
 
         normalized = self.normalizer.rewrite(word)
         unit_div_info = "*,*"
-        if (udm := pos.get("unit_div_mode")) != None:
-            unit_div_info = self.split(normalized, udm)
+
+        try:
+            if (udm := pos.get("unit_div_mode")) != None:
+                unit_div_info = self.split(normalized, udm)
+        except OOVError as e:
+            print(e)
+            raise e
 
         split_mode = pos.get("split_mode", "*")
         return f"{normalized},{pos['left_id']},{pos['right_id']},{pos['cost']},{word},{pos['sudachi_pos']},{yomi},{word},*,{split_mode},{unit_div_info},*"
@@ -133,24 +148,40 @@ class Converter:
     def split(self, normalized: str, udm: list[str]) -> str:
         unit_div_info = []
         if "A" in udm:
-            words = [
-                f'{m.surface()},{",".join(m.part_of_speech())},{m.reading_form()}'
-                for m in self.tokenizer.tokenize(
-                    normalized, tokenizer.Tokenizer.SplitMode.A
-                )
-            ]
+            words = []
+            oov = []
+            for m in self.tokenizer.tokenize(normalized, tokenizer.Tokenizer.SplitMode.A):
+                if m.is_oov():
+                    oov.append(m.surface())
+                    continue
+                # if m.dictionary_id():
+                #     info = f'U{m.word_id()}'
+
+                info = f'{m.surface()},{",".join(m.part_of_speech())},{m.reading_form()}'
+
+                words.append(info)
+
+            if len(oov) > 0:
+                raise OOVError(f"split word has out of vocab: {oov} in {normalized}")
+
             info = "/".join(words)
             unit_div_info.append(f'"{info}"')
         else:
             unit_div_info.append("*")
 
         if "B" in udm:
-            words = [
-                f'{m.surface()},{",".join(m.part_of_speech())},{m.reading_form()}'
-                for m in self.tokenizer.tokenize(
-                    normalized, tokenizer.Tokenizer.SplitMode.B
-                )
-            ]
+            words = []
+            oov = []
+            for m in self.tokenizer.tokenize(normalized, tokenizer.Tokenizer.SplitMode.B):
+                if m.is_oov():
+                    oov.append(m.surface())
+                    continue
+                info = f'{m.surface()},{",".join(m.part_of_speech())},{m.reading_form()}'
+                words.append(info)
+
+            if len(oov) > 0:
+                raise OOVError(f"split word has out of vocab: {oov} in {normalized}")
+
             info = "/".join(words)
             unit_div_info.append(f'"{info}"')
         else:
@@ -167,6 +198,7 @@ def cli() -> str:
     config = args.config
     sudachi_setting = args.sudachi_setting
     sudachi_dict_type = args.sudachi_dict_type
+    merge_dict = args.merge_dict
 
     c = Converter(
         rewrite,
@@ -176,6 +208,11 @@ def cli() -> str:
         rm=rm,
     )
 
+    with fileinput.input(files=merge_dict) as merged:
+        for line in merged:
+            line = line.replace("\n" , "")
+            out.write(f"{line}\n")
+
     with fileinput.input(files=args.file) as input:
         for line in input:
             line = line.strip()
@@ -189,10 +226,10 @@ def cli() -> str:
                 converted = c.convert(line)
                 if converted == "":
                     continue
-            except (UnSupportedPosError, DictFormatError) as e:
+            except (UnSupportedPosError, DictFormatError, OOVError) as e:
                 if args.ignore:
                     continue
                 else:
                     raise e
-
+            print(converted)
             out.write(f"{converted}\n")
